@@ -363,7 +363,10 @@ static void
 size_allocate(GtkWidget *widget, GdkRectangle *allocation, gpointer user_data)
 {
     GeglGtkView *self = GEGL_GTK_VIEW(widget);
-    view_helper_set_allocation(GET_PRIVATE(self), allocation);
+    gint scale_factor;
+
+    scale_factor = gtk_widget_get_scale_factor (widget);
+    view_helper_set_allocation(GET_PRIVATE(self), allocation, scale_factor);
 }
 
 static void
@@ -452,14 +455,6 @@ gegl_gtk_view_new(void)
     return GEGL_GTK_VIEW(g_object_new(GEGL_GTK_TYPE_VIEW, NULL));
 }
 
-GeglGtkView *
-gegl_gtk_view_new_for_buffer(GeglBuffer *buffer)
-{
-    GeglNode *node = gegl_node("gegl:buffer-source",
-                     "buffer", buffer, NULL);
-    return gegl_gtk_view_new_for_node(node);
-}
-
 /**
  * gegl_gtk_view_new_for_node:
  * @node: The #GeglNode to display
