@@ -34,7 +34,6 @@
 #include <grilo.h>
 #include <libgnome-desktop/gnome-bg.h>
 
-#include "eog-debug.h"
 #include "photos-application.h"
 #include "photos-base-item.h"
 #include "photos-camera-cache.h"
@@ -61,18 +60,25 @@
 struct _PhotosApplicationPrivate
 {
   GCancellable *create_window_cancellable;
+  GHashTable *refresh_miner_ids;
   GList *miners;
   GList *miners_running;
   GResource *resource;
   GSettings *bg_settings;
   GSettings *ss_settings;
+  GSimpleAction *brightness_contrast_action;
+  GSimpleAction *crop_action;
+  GSimpleAction *denoise_action;
+  GSimpleAction *edit_cancel_action;
   GSimpleAction *fs_action;
   GSimpleAction *gear_action;
+  GSimpleAction *insta_action;
   GSimpleAction *load_next_action;
   GSimpleAction *load_previous_action;
   GSimpleAction *open_action;
   GSimpleAction *print_action;
   GSimpleAction *properties_action;
+  GSimpleAction *save_action;
   GSimpleAction *search_action;
   GSimpleAction *search_match_action;
   GSimpleAction *search_source_action;
@@ -81,6 +87,8 @@ struct _PhotosApplicationPrivate
   GSimpleAction *sel_none_action;
   GSimpleAction *set_bg_action;
   GSimpleAction *set_ss_action;
+  GSimpleAction *sharpen_action;
+  GSimpleAction *undo_action;
   GtkWidget *main_window;
   PhotosCameraCache *camera_cache;
   PhotosSearchContextState *state;
@@ -138,7 +146,7 @@ struct _PhotosApplicationRefreshData
   GomMiner *miner;
 };
 
-static gboolean photos_application_refresh_miner_now (PhotosApplication *self, GomMiner *miner);
+static void photos_application_refresh_miner_now (PhotosApplication *self, GomMiner *miner);
 static void photos_application_start_miners (PhotosApplication *self);
 static void photos_application_start_miners_second (PhotosApplication *self);
 static void photos_application_stop_miners (PhotosApplication *self);
@@ -152,7 +160,8 @@ photos_application_create_data_new (PhotosApplication *application,
   PhotosApplicationCreateData *data;
 
   data = g_slice_new0 (PhotosApplicationCreateData);
-  data->application = g_object_ref (application);
+  g_application_hold (G_APPLICATION (application));
+  data->application = application;
   data->extension_name = g_strdup (extension_name);
   data->miner_name = g_strdup (miner_name);
   return data;
@@ -162,7 +171,7 @@ photos_application_create_data_new (PhotosApplication *application,
 static void
 photos_application_create_data_free (PhotosApplicationCreateData *data)
 {
-  g_object_unref (data->application);
+  g_application_release (G_APPLICATION (data->application));
   g_free (data->extension_name);
   g_free (data->miner_name);
   g_slice_free (PhotosApplicationCreateData, data);
@@ -175,7 +184,8 @@ photos_application_refresh_data_new (PhotosApplication *application, GomMiner *m
   PhotosApplicationRefreshData *data;
 
   data = g_slice_new0 (PhotosApplicationRefreshData);
-  data->application = g_object_ref (application);
+  g_application_hold (G_APPLICATION (application));
+  data->application = application;
   data->miner = g_object_ref (miner);
   return data;
 }
@@ -184,7 +194,7 @@ photos_application_refresh_data_new (PhotosApplication *application, GomMiner *m
 static void
 photos_application_refresh_data_free (PhotosApplicationRefreshData *data)
 {
-  g_object_unref (data->application);
+  g_application_release (G_APPLICATION (data->application));
   g_object_unref (data->miner);
   g_slice_free (PhotosApplicationRefreshData, data);
 }
@@ -240,7 +250,7 @@ photos_application_tracker_clear_rdf_types (GObject *source_object, GAsyncResult
     }
 
  out:
-  g_object_unref (self);
+  g_application_release (G_APPLICATION (self));
 }
 
 
@@ -248,9 +258,20 @@ static void
 photos_application_destroy (PhotosApplication *self)
 {
   PhotosApplicationPrivate *priv = self->priv;
+  GHashTableIter iter;
+  gpointer refresh_miner_id_data;
 
   priv->main_window = NULL;
 
+  g_hash_table_iter_init (&iter, priv->refresh_miner_ids);
+  while (g_hash_table_iter_next (&iter, NULL, &refresh_miner_id_data))
+    {
+      guint refresh_miner_id = GPOINTER_TO_UINT (refresh_miner_id_data);
+      g_source_remove (refresh_miner_id);
+    }
+
+  g_hash_table_remove_all (priv->refresh_miner_ids);
+
   g_cancellable_cancel (priv->create_window_cancellable);
   g_clear_object (&priv->create_window_cancellable);
   priv->create_window_cancellable = g_cancellable_new ();
@@ -259,10 +280,11 @@ photos_application_destroy (PhotosApplication *self)
 
   if (priv->extract_priority != NULL)
     {
+      g_application_hold (G_APPLICATION (self));
       tracker_extract_priority_call_clear_rdf_types (priv->extract_priority,
                                                      NULL,
                                                      photos_application_tracker_clear_rdf_types,
-                                                     g_object_ref (self));
+                                                     self);
       g_clear_object (&priv->extract_priority);
     }
 }
@@ -363,7 +385,7 @@ photos_application_tracker_set_rdf_types (GObject *source_object, GAsyncResult *
     }
 
  out:
-  g_object_unref (self);
+  g_application_release (G_APPLICATION (self));
 }
 
 
@@ -385,14 +407,15 @@ photos_application_tracker_extract_priority (GObject *source_object, GAsyncResul
       goto out;
     }
 
+  g_application_hold (G_APPLICATION (self));
   tracker_extract_priority_call_set_rdf_types (priv->extract_priority,
                                                rdf_types,
                                                priv->create_window_cancellable,
                                                photos_application_tracker_set_rdf_types,
-                                               g_object_ref (self));
+                                               self);
 
  out:
-  g_object_unref (self);
+  g_application_release (G_APPLICATION (self));
 }
 
 
@@ -407,13 +430,14 @@ photos_application_create_window (PhotosApplication *self)
   priv->main_window = photos_main_window_new (GTK_APPLICATION (self));
   g_signal_connect_swapped (priv->main_window, "destroy", G_CALLBACK (photos_application_destroy), self);
 
+  g_application_hold (G_APPLICATION (self));
   tracker_extract_priority_proxy_new_for_bus (G_BUS_TYPE_SESSION,
                                               G_DBUS_PROXY_FLAGS_NONE,
                                               "org.freedesktop.Tracker1.Miner.Extract",
                                               "/org/freedesktop/Tracker1/Extract/Priority",
                                               priv->create_window_cancellable,
                                               photos_application_tracker_extract_priority,
-                                              g_object_ref (self));
+                                              self);
 
   photos_application_start_miners (self);
 }
@@ -451,7 +475,7 @@ photos_application_activate_query_executed (TrackerSparqlCursor *cursor, gpointe
   photos_application_activate_item (self, item);
 
  out:
-  g_object_unref (self);
+  g_application_release (G_APPLICATION (self));
 }
 
 
@@ -474,11 +498,12 @@ photos_application_activate_result (PhotosApplication *self,
       PhotosSingleItemJob *job;
 
       job = photos_single_item_job_new (identifier);
+      g_application_hold (G_APPLICATION (self));
       photos_single_item_job_run (job,
                                   priv->state,
                                   PHOTOS_QUERY_FLAGS_UNFILTERED,
                                   photos_application_activate_query_executed,
-                                  g_object_ref (self));
+                                  self);
       g_object_unref (job);
     }
 }
@@ -495,6 +520,57 @@ photos_application_can_fullscreen_changed (PhotosApplication *self)
 }
 
 
+static void
+photos_application_edit_cancel_process (GObject *source_object, GAsyncResult *res, gpointer user_data)
+{
+  PhotosApplication *self = PHOTOS_APPLICATION (user_data);
+  GError *error = NULL;
+  PhotosBaseItem *item = PHOTOS_BASE_ITEM (source_object);
+
+  photos_base_item_process_finish (item, res, &error);
+  if (error != NULL)
+    {
+      g_warning ("Unable to process item: %s", error->message);
+      g_error_free (error);
+    }
+
+  /* Go back, no matter what. */
+  photos_mode_controller_go_back (self->priv->state->mode_cntrlr);
+  g_application_release (G_APPLICATION (self));
+}
+
+
+static void
+photos_application_edit_cancel (PhotosApplication *self)
+{
+  PhotosApplicationPrivate *priv = self->priv;
+  PhotosBaseItem *item;
+
+  item = PHOTOS_BASE_ITEM (photos_base_manager_get_active_object (priv->state->item_mngr));
+  g_return_if_fail (item != NULL);
+
+  while (photos_base_item_operation_undo (item))
+    ;
+
+  g_application_hold (G_APPLICATION (self));
+  photos_base_item_process_async (item, NULL, photos_application_edit_cancel_process, self);
+}
+
+
+static void
+photos_application_edit_current (PhotosApplication *self)
+{
+  PhotosApplicationPrivate *priv = self->priv;
+  PhotosBaseItem *item;
+
+  item = PHOTOS_BASE_ITEM (photos_base_manager_get_active_object (priv->state->item_mngr));
+  if (item == NULL)
+    return;
+
+  photos_mode_controller_set_window_mode (priv->state->mode_cntrlr, PHOTOS_WINDOW_MODE_EDIT);
+}
+
+
 static void
 photos_application_fullscreen (PhotosApplication *self, GVariant *parameter)
 {
@@ -589,7 +665,11 @@ static gboolean
 photos_application_refresh_miner_timeout (gpointer user_data)
 {
   PhotosApplicationRefreshData *data = (PhotosApplicationRefreshData *) user_data;
-  return photos_application_refresh_miner_now (data->application, data->miner);
+  PhotosApplication *self = data->application;
+
+  g_hash_table_remove (self->priv->refresh_miner_ids, data->miner);
+  photos_application_refresh_miner_now (self, data->miner);
+  return G_SOURCE_REMOVE;
 }
 
 
@@ -601,6 +681,11 @@ photos_application_refresh_db (GObject *source_object, GAsyncResult *res, gpoint
   GError *error;
   GomMiner *miner = GOM_MINER (source_object);
   PhotosApplicationRefreshData *data;
+  gpointer refresh_miner_id_data;
+  guint refresh_miner_id;
+
+  refresh_miner_id_data = g_hash_table_lookup (priv->refresh_miner_ids, miner);
+  g_assert_null (refresh_miner_id_data);
 
   priv->miners_running = g_list_remove (priv->miners_running, miner);
   g_signal_emit (self, signals[MINERS_CHANGED], 0, priv->miners_running);
@@ -615,19 +700,20 @@ photos_application_refresh_db (GObject *source_object, GAsyncResult *res, gpoint
     }
 
   data = photos_application_refresh_data_new (self, miner);
-  g_timeout_add_seconds_full (G_PRIORITY_DEFAULT,
-                              MINER_REFRESH_TIMEOUT,
-                              photos_application_refresh_miner_timeout,
-                              data,
-                              (GDestroyNotify) photos_application_refresh_data_free);
+  refresh_miner_id = g_timeout_add_seconds_full (G_PRIORITY_DEFAULT,
+                                                 MINER_REFRESH_TIMEOUT,
+                                                 photos_application_refresh_miner_timeout,
+                                                 data,
+                                                 (GDestroyNotify) photos_application_refresh_data_free);
+  g_hash_table_insert (priv->refresh_miner_ids, miner, GUINT_TO_POINTER (refresh_miner_id));
 
  out:
-  g_object_unref (self);
+  g_application_release (G_APPLICATION (self));
   g_object_unref (miner);
 }
 
 
-static gboolean
+static void
 photos_application_refresh_miner_now (PhotosApplication *self, GomMiner *miner)
 {
   PhotosApplicationPrivate *priv = self->priv;
@@ -635,17 +721,15 @@ photos_application_refresh_miner_now (PhotosApplication *self, GomMiner *miner)
   const gchar *const index_types[] = {"photos", NULL};
 
   if (g_getenv ("GNOME_PHOTOS_DISABLE_MINERS") != NULL)
-    goto out;
+    return;
 
   priv->miners_running = g_list_prepend (priv->miners_running, g_object_ref (miner));
   g_signal_emit (self, signals[MINERS_CHANGED], 0, priv->miners_running);
 
   cancellable = g_cancellable_new ();
   g_object_set_data_full (G_OBJECT (miner), "cancellable", cancellable, g_object_unref);
-  gom_miner_call_refresh_db (miner, index_types, cancellable, photos_application_refresh_db, g_object_ref (self));
-
- out:
-  return G_SOURCE_REMOVE;
+  g_application_hold (G_APPLICATION (self));
+  gom_miner_call_refresh_db (miner, index_types, cancellable, photos_application_refresh_db, self);
 }
 
 
@@ -692,6 +776,98 @@ photos_application_quit (PhotosApplication *self, GVariant *parameter)
 }
 
 
+static void
+photos_application_save_save (GObject *source_object, GAsyncResult *res, gpointer user_data)
+{
+  PhotosApplication *self = PHOTOS_APPLICATION (user_data);
+  PhotosBaseItem *item = PHOTOS_BASE_ITEM (source_object);
+  GError *error = NULL;
+
+  photos_base_item_save_finish (item, res, &error);
+  if (error != NULL)
+    {
+      g_warning ("Unable to save: %s", error->message);
+      g_error_free (error);
+      goto out;
+    }
+
+  photos_mode_controller_go_back (self->priv->state->mode_cntrlr);
+
+ out:
+  g_application_release (G_APPLICATION (self));
+}
+
+
+static void
+photos_application_save (PhotosApplication *self)
+{
+  PhotosApplicationPrivate *priv = self->priv;
+  GDateTime *now = NULL;
+  GError *error = NULL;
+  GFile *parent = NULL;
+  PhotosBaseItem *item;
+  const gchar *mime_type;
+  const gchar *pictures_path;
+  gchar *filename = NULL;
+  gchar *extension = NULL;
+  gchar *origin = NULL;
+  gchar *parent_path = NULL;
+  gchar *path = NULL;
+  gchar *type = NULL;
+  gchar *uri = NULL;
+
+  item = PHOTOS_BASE_ITEM (photos_base_manager_get_active_object (priv->state->item_mngr));
+  g_return_if_fail (item != NULL);
+
+  pictures_path = g_get_user_special_dir (G_USER_DIRECTORY_PICTURES);
+  parent_path = g_build_filename (pictures_path, PACKAGE_TARNAME "-exported", NULL);
+  parent = g_file_new_for_path (parent_path);
+  g_file_make_directory_with_parents (parent, NULL, &error);
+  if (error != NULL)
+    {
+      if (g_error_matches (error, G_IO_ERROR, G_IO_ERROR_EXISTS))
+        {
+          g_error_free (error);
+          goto carry_on;
+        }
+      else
+        {
+          g_warning ("Unable to create %s: %s", parent_path, error->message);
+          g_error_free (error);
+          goto out;
+        }
+    }
+
+ carry_on:
+  now = g_date_time_new_now_local ();
+  origin = g_date_time_format (now, "%Y-%m-%d %H-%M-%S");
+
+  mime_type = photos_base_item_get_mime_type (item);
+  extension = photos_utils_get_extension_from_mime_type (mime_type);
+
+  filename = g_strdup_printf ("%s.%s", origin, extension);
+  path = g_build_filename (parent_path, filename, NULL);
+  uri = photos_utils_convert_path_to_uri (path);
+
+  type = photos_utils_get_pixbuf_type_from_mime_type (mime_type);
+
+  g_application_hold (G_APPLICATION (self));
+  photos_base_item_save_async (item, uri, type, NULL, photos_application_save_save, self);
+
+ out:
+  if (now != NULL)
+    g_date_time_unref (now);
+
+  g_free (filename);
+  g_free (origin);
+  g_free (parent_path);
+  g_free (path);
+  g_free (type);
+  g_free (uri);
+  g_clear_object (&parent);
+}
+
+
 static void
 photos_application_set_bg_common_download (GObject *source_object, GAsyncResult *res, gpointer user_data)
 {
@@ -841,6 +1017,16 @@ photos_application_window_mode_changed (PhotosApplication *self, PhotosWindowMod
   PhotosApplicationPrivate *priv = self->priv;
   gboolean enable;
 
+  enable = (mode == PHOTOS_WINDOW_MODE_EDIT);
+  g_simple_action_set_enabled (priv->brightness_contrast_action, enable);
+  g_simple_action_set_enabled (priv->crop_action, enable);
+  g_simple_action_set_enabled (priv->denoise_action, enable);
+  g_simple_action_set_enabled (priv->edit_cancel_action, enable);
+  g_simple_action_set_enabled (priv->insta_action, enable);
+  g_simple_action_set_enabled (priv->save_action, enable);
+  g_simple_action_set_enabled (priv->sharpen_action, enable);
+  g_simple_action_set_enabled (priv->undo_action, enable);
+
   enable = (mode == PHOTOS_WINDOW_MODE_COLLECTIONS
             || mode == PHOTOS_WINDOW_MODE_FAVORITES
             || mode == PHOTOS_WINDOW_MODE_OVERVIEW
@@ -941,6 +1127,14 @@ photos_application_dbus_unregister (GApplication *application,
 static void
 photos_application_shutdown (GApplication *application)
 {
+  PhotosApplication *self = PHOTOS_APPLICATION (application);
+  PhotosApplicationPrivate *priv = self->priv;
+  guint refresh_miner_ids_size;
+
+  refresh_miner_ids_size = g_hash_table_size (priv->refresh_miner_ids);
+  g_assert (refresh_miner_ids_size == 0);
+
+  g_clear_pointer (&priv->refresh_miner_ids, (GDestroyNotify) g_hash_table_unref);
   gegl_exit ();
 
   G_APPLICATION_CLASS (photos_application_parent_class)->shutdown (application);
@@ -955,10 +1149,11 @@ photos_application_startup (GApplication *application)
   GError *error;
   GSimpleAction *action;
   GrlRegistry *registry;
+  GtkIconTheme *icon_theme;
   GtkSettings *settings;
   GVariant *state;
   GVariantType *parameter_type;
-  const gchar *delete_accels[2] = {"Delete", "KP_Delete", NULL};
+  const gchar *delete_accels[3] = {"Delete", "KP_Delete", NULL};
   const gchar *fullscreen_accels[2] = {"F11", NULL};
   const gchar *gear_menu_accels[2] = {"F10", NULL};
   const gchar *print_current_accels[2] = {"<Primary>p", NULL};
@@ -980,6 +1175,7 @@ photos_application_startup (GApplication *application)
     }
 
   priv->create_window_cancellable = g_cancellable_new ();
+  priv->refresh_miner_ids = g_hash_table_new (g_direct_hash, g_direct_equal);
 
   priv->bg_settings = g_settings_new (DESKTOP_BACKGROUND_SCHEMA);
   priv->ss_settings = g_settings_new (DESKTOP_SCREENSAVER_SCHEMA);
@@ -987,6 +1183,9 @@ photos_application_startup (GApplication *application)
   priv->resource = photos_get_resource ();
   g_resources_register (priv->resource);
 
+  icon_theme = gtk_icon_theme_get_default ();
+  gtk_icon_theme_add_resource_path (icon_theme, "/org/gnome/Photos/icons");
+
   settings = gtk_settings_get_default ();
   g_object_set (settings, "gtk-application-prefer-dark-theme", TRUE, NULL);
   g_signal_connect (settings, "notify::gtk-theme-name", G_CALLBACK (photos_application_theme_changed), NULL);
@@ -1002,10 +1201,35 @@ photos_application_startup (GApplication *application)
   g_action_map_add_action (G_ACTION_MAP (self), G_ACTION (action));
   g_object_unref (action);
 
+  parameter_type = g_variant_type_new ("a{sd}");
+  priv->brightness_contrast_action = g_simple_action_new ("brightness-contrast-current", parameter_type);
+  g_action_map_add_action (G_ACTION_MAP (self), G_ACTION (priv->brightness_contrast_action));
+  g_variant_type_free (parameter_type);
+
+  parameter_type = g_variant_type_new ("a{sd}");
+  priv->crop_action = g_simple_action_new ("crop-current", parameter_type);
+  g_action_map_add_action (G_ACTION_MAP (self), G_ACTION (priv->crop_action));
+  g_variant_type_free (parameter_type);
+
   action = g_simple_action_new ("delete", NULL);
   g_action_map_add_action (G_ACTION_MAP (self), G_ACTION (action));
   g_object_unref (action);
 
+  priv->denoise_action = g_simple_action_new ("denoise-current", G_VARIANT_TYPE_UINT16);
+  g_action_map_add_action (G_ACTION_MAP (self), G_ACTION (priv->denoise_action));
+
+  priv->edit_cancel_action = g_simple_action_new ("edit-cancel", NULL);
+  g_signal_connect_swapped (priv->edit_cancel_action,
+                            "activate",
+                            G_CALLBACK (photos_application_edit_cancel),
+                            self);
+  g_action_map_add_action (G_ACTION_MAP (self), G_ACTION (priv->edit_cancel_action));
+
+  action = g_simple_action_new ("edit-current", NULL);
+  g_signal_connect_swapped (action, "activate", G_CALLBACK (photos_application_edit_current), self);
+  g_action_map_add_action (G_ACTION_MAP (self), G_ACTION (action));
+  g_object_unref (action);
+
   priv->fs_action = g_simple_action_new ("fullscreen", NULL);
   g_signal_connect_swapped (priv->fs_action, "activate", G_CALLBACK (photos_application_fullscreen), self);
   g_action_map_add_action (G_ACTION_MAP (self), G_ACTION (priv->fs_action));
@@ -1020,6 +1244,9 @@ photos_application_startup (GApplication *application)
   g_signal_connect (priv->gear_action, "activate", G_CALLBACK (photos_application_action_toggle), self);
   g_action_map_add_action (G_ACTION_MAP (self), G_ACTION (priv->gear_action));
 
+  priv->insta_action = g_simple_action_new ("insta-current", G_VARIANT_TYPE_INT16);
+  g_action_map_add_action (G_ACTION_MAP (self), G_ACTION (priv->insta_action));
+
   priv->load_next_action = g_simple_action_new ("load-next", NULL);
   g_action_map_add_action (G_ACTION_MAP (self), G_ACTION (priv->load_next_action));
 
@@ -1048,6 +1275,10 @@ photos_application_startup (GApplication *application)
   g_action_map_add_action (G_ACTION_MAP (self), G_ACTION (action));
   g_object_unref (action);
 
+  priv->save_action = g_simple_action_new ("save-current", NULL);
+  g_signal_connect_swapped (priv->save_action, "activate", G_CALLBACK (photos_application_save), self);
+  g_action_map_add_action (G_ACTION_MAP (self), G_ACTION (priv->save_action));
+
   state = g_variant_new ("b", FALSE);
   priv->search_action = g_simple_action_new_stateful ("search", NULL, state);
   g_signal_connect (priv->search_action, "activate", G_CALLBACK (photos_application_action_toggle), self);
@@ -1093,6 +1324,12 @@ photos_application_startup (GApplication *application)
   g_signal_connect_swapped (priv->set_ss_action, "activate", G_CALLBACK (photos_application_set_bg_common), self);
   g_action_map_add_action (G_ACTION_MAP (self), G_ACTION (priv->set_ss_action));
 
+  priv->sharpen_action = g_simple_action_new ("sharpen-current", G_VARIANT_TYPE_DOUBLE);
+  g_action_map_add_action (G_ACTION_MAP (self), G_ACTION (priv->sharpen_action));
+
+  priv->undo_action = g_simple_action_new ("undo-current", NULL);
+  g_action_map_add_action (G_ACTION_MAP (self), G_ACTION (priv->undo_action));
+
   g_signal_connect_swapped (priv->state->mode_cntrlr,
                             "window-mode-changed",
                             G_CALLBACK (photos_application_window_mode_changed),
@@ -1159,13 +1396,19 @@ photos_application_dispose (GObject *object)
   g_clear_object (&priv->create_window_cancellable);
   g_clear_object (&priv->bg_settings);
   g_clear_object (&priv->ss_settings);
+  g_clear_object (&priv->brightness_contrast_action);
+  g_clear_object (&priv->crop_action);
+  g_clear_object (&priv->denoise_action);
+  g_clear_object (&priv->edit_cancel_action);
   g_clear_object (&priv->fs_action);
   g_clear_object (&priv->gear_action);
+  g_clear_object (&priv->insta_action);
   g_clear_object (&priv->load_next_action);
   g_clear_object (&priv->load_previous_action);
   g_clear_object (&priv->open_action);
   g_clear_object (&priv->print_action);
   g_clear_object (&priv->properties_action);
+  g_clear_object (&priv->save_action);
   g_clear_object (&priv->search_action);
   g_clear_object (&priv->search_match_action);
   g_clear_object (&priv->search_source_action);
@@ -1174,6 +1417,8 @@ photos_application_dispose (GObject *object)
   g_clear_object (&priv->sel_none_action);
   g_clear_object (&priv->set_bg_action);
   g_clear_object (&priv->set_ss_action);
+  g_clear_object (&priv->sharpen_action);
+  g_clear_object (&priv->undo_action);
   g_clear_object (&priv->camera_cache);
   g_clear_object (&priv->extract_priority);
 
@@ -1191,11 +1436,8 @@ static void
 photos_application_finalize (GObject *object)
 {
   PhotosApplication *self = PHOTOS_APPLICATION (object);
-  PhotosApplicationPrivate *priv = self->priv;
 
-  if (G_UNLIKELY (priv->create_miners_count != 0))
-    g_critical ("Application is being destroyed while %u miner proxies are still being created.",
-                priv->create_miners_count);
+  g_assert (self->priv->create_miners_count == 0);
 
   G_OBJECT_CLASS (photos_application_parent_class)->finalize (object);
 }
@@ -1209,8 +1451,6 @@ photos_application_init (PhotosApplication *self)
   self->priv = photos_application_get_instance_private (self);
   priv = self->priv;
 
-  eog_debug_init ();
-
   photos_utils_ensure_builtins ();
 
   priv->search_provider = photos_search_provider_new ();
