@@ -26,6 +26,7 @@
 
 #include "config.h"
 
+#include <stdarg.h>
 #include <string.h>
 
 #include <gdk/gdk.h>
@@ -38,10 +39,12 @@
 #include "photos-application.h"
 #include "photos-base-item.h"
 #include "photos-collection-icon-watcher.h"
+#include "photos-debug.h"
 #include "photos-delete-item-job.h"
 #include "photos-filterable.h"
 #include "photos-icons.h"
 #include "photos-local-item.h"
+#include "photos-pipeline.h"
 #include "photos-print-notification.h"
 #include "photos-print-operation.h"
 #include "photos-query.h"
@@ -55,14 +58,19 @@ struct _PhotosBaseItemPrivate
 {
   cairo_surface_t *surface;
   GdkPixbuf *original_icon;
-  GeglNode *graph;
-  GeglNode *node;
-  GeglRectangle bbox;
+  GeglNode *buffer_sink;
+  GeglNode *buffer_source;
+  GeglNode *edit_graph;
+  GeglNode *load_graph;
+  GeglNode *load;
+  GeglProcessor *processor;
   GMutex mutex_download;
   GMutex mutex;
   GQuark equipment;
   GQuark flash;
+  GQuark orientation;
   PhotosCollectionIconWatcher *watcher;
+  PhotosPipeline *pipeline;
   PhotosSelectionController *sel_cntrlr;
   TrackerSparqlCursor *cursor;
   gboolean collection;
@@ -225,7 +233,7 @@ photos_base_item_check_effects_and_update_info (PhotosBaseItem *self)
       emblem_icons = g_list_prepend (emblem_icons, pix);
     }
 
-  if (g_list_length (emblem_icons) > 0)
+  if (emblem_icons != NULL)
     {
       GIcon *emblemed_icon;
       GList *l;
@@ -331,20 +339,20 @@ photos_base_item_create_thumbnail_in_thread_func (gpointer data, gpointer user_d
   PhotosBaseItem *self;
   GCancellable *cancellable;
   GError *error;
-  gboolean op_res;
+  gboolean result;
 
   self = PHOTOS_BASE_ITEM (g_task_get_source_object (task));
   cancellable = g_task_get_cancellable (task);
 
   error = NULL;
-  op_res = PHOTOS_BASE_ITEM_GET_CLASS (self)->create_thumbnail (self, cancellable, &error);
+  result = PHOTOS_BASE_ITEM_GET_CLASS (self)->create_thumbnail (self, cancellable, &error);
   if (error != NULL)
     {
       g_task_return_error (task, error);
       goto out;
     }
 
-  g_task_return_boolean (task, op_res);
+  g_task_return_boolean (task, result);
 
  out:
   g_object_unref (task);
@@ -381,7 +389,6 @@ photos_base_item_create_thumbnail_async (PhotosBaseItem *self,
   GTask *task;
 
   task = g_task_new (self, cancellable, callback, user_data);
-  g_task_set_check_cancellable (task, TRUE);
   g_task_set_source_tag (task, photos_base_item_create_thumbnail_async);
 
   g_thread_pool_push (create_thumbnail_pool, g_object_ref (task), NULL);
@@ -733,29 +740,20 @@ photos_base_item_file_query_info (GObject *source_object, GAsyncResult *res, gpo
 }
 
 
-static GeglNode *
-photos_base_item_load (PhotosBaseItem *self, GCancellable *cancellable, GError **error)
+static GeglBuffer *
+photos_base_item_load_buffer (PhotosBaseItem *self, GCancellable *cancellable, GError **error)
 {
   PhotosBaseItemPrivate *priv = self->priv;
-  GeglNode *ret_val = NULL;
+  GeglBuffer *ret_val = NULL;
   gchar *path = NULL;
 
-  if (priv->graph == NULL)
-    {
-      path = photos_base_item_download (self, cancellable, error);
-      if (path == NULL)
-        goto out;
+  path = photos_base_item_download (self, cancellable, error);
+  if (path == NULL)
+    goto out;
 
-      priv->graph = gegl_node_new ();
-      priv->node = gegl_node_new_child (priv->graph,
-                                        "operation", "gegl:load",
-                                        "path", path,
-                                        NULL);
-    }
-
-  gegl_node_process (priv->node);
-  priv->bbox = gegl_node_get_bounding_box (priv->node);
-  ret_val = g_object_ref (priv->node);
+  gegl_node_set (priv->load, "path", path, NULL);
+  gegl_node_set (priv->buffer_sink, "buffer", &ret_val, NULL);
+  gegl_node_process (priv->buffer_sink);
 
  out:
   g_free (path);
@@ -764,32 +762,262 @@ photos_base_item_load (PhotosBaseItem *self, GCancellable *cancellable, GError *
 
 
 static void
-photos_base_item_load_in_thread_func (GTask *task,
-                                      gpointer source_object,
-                                      gpointer task_data,
-                                      GCancellable *cancellable)
+photos_base_item_load_buffer_in_thread_func (GTask *task,
+                                             gpointer source_object,
+                                             gpointer task_data,
+                                             GCancellable *cancellable)
 {
   PhotosBaseItem *self = PHOTOS_BASE_ITEM (source_object);
   PhotosBaseItemPrivate *priv = self->priv;
-  GeglNode *node;
+  GeglBuffer *buffer;
   GError *error = NULL;
 
   g_mutex_lock (&priv->mutex);
 
-  node = photos_base_item_load (self, cancellable, &error);
+  buffer = photos_base_item_load_buffer (self, cancellable, &error);
   if (error != NULL)
     {
       g_task_return_error (task, error);
       goto out;
     }
 
-  g_task_return_pointer (task, node, g_object_unref);
+  g_task_return_pointer (task, buffer, g_object_unref);
 
  out:
   g_mutex_unlock (&priv->mutex);
 }
 
 
+static void
+photos_base_item_load_buffer_async (PhotosBaseItem *self,
+                                    GCancellable *cancellable,
+                                    GAsyncReadyCallback callback,
+                                    gpointer user_data)
+{
+  PhotosBaseItemPrivate *priv = self->priv;
+  GTask *task;
+
+  g_return_if_fail (PHOTOS_IS_BASE_ITEM (self));
+
+  if (priv->load_graph == NULL)
+    {
+      GeglNode *orientation;
+
+      priv->load_graph = gegl_node_new ();
+      priv->load = gegl_node_new_child (priv->load_graph, "operation", "gegl:load", NULL);
+      orientation = photos_utils_create_orientation_node (priv->load_graph, priv->orientation);
+      priv->buffer_sink = gegl_node_new_child (priv->load_graph, "operation", "gegl:buffer-sink", NULL);
+      gegl_node_link_many (priv->load, orientation, priv->buffer_sink, NULL);
+    }
+
+  if (priv->edit_graph == NULL)
+    {
+      GeglNode *graph;
+
+      priv->edit_graph = gegl_node_new ();
+      priv->buffer_source = gegl_node_new_child (priv->edit_graph, "operation", "gegl:buffer-source", NULL);
+      priv->pipeline = photos_pipeline_new (priv->edit_graph);
+      graph = photos_pipeline_get_graph (priv->pipeline);
+      gegl_node_link (priv->buffer_source, graph);
+    }
+
+  task = g_task_new (self, cancellable, callback, user_data);
+  g_task_set_source_tag (task, photos_base_item_load_buffer_async);
+
+  g_task_run_in_thread (task, photos_base_item_load_buffer_in_thread_func);
+  g_object_unref (task);
+}
+
+
+static GeglBuffer *
+photos_base_item_load_buffer_finish (PhotosBaseItem *self, GAsyncResult *res, GError **error)
+{
+  GTask *task = G_TASK (res);
+
+  g_return_val_if_fail (g_task_is_valid (res, self), NULL);
+  g_return_val_if_fail (g_task_get_source_tag (task) == photos_base_item_load_buffer_async, NULL);
+  g_return_val_if_fail (error == NULL || *error == NULL, NULL);
+
+  return g_task_propagate_pointer (task, error);
+}
+
+
+static void
+photos_base_item_load_process (GObject *source_object, GAsyncResult *res, gpointer user_data)
+{
+  GTask *task = G_TASK (user_data);
+  PhotosBaseItem *self;
+  GeglNode *graph;
+  GError *error;
+
+  self = PHOTOS_BASE_ITEM (g_task_get_source_object (task));
+
+  error = NULL;
+  photos_base_item_process_finish (self, res, &error);
+  if (error != NULL)
+    {
+      g_task_return_error (task, error);
+      goto out;
+    }
+
+  graph = photos_pipeline_get_graph (self->priv->pipeline);
+  g_task_return_pointer (task, g_object_ref (graph), g_object_unref);
+
+ out:
+  g_object_unref (task);
+}
+
+
+static void
+photos_base_item_load_load_buffer (GObject *source_object, GAsyncResult *res, gpointer user_data)
+{
+  GTask *task = G_TASK (user_data);
+  PhotosBaseItem *self;
+  PhotosBaseItemPrivate *priv;
+  const Babl *format;
+  GCancellable *cancellable;
+  GeglBuffer *buffer = NULL;
+  GError *error;
+  const gchar *format_name;
+
+  self = PHOTOS_BASE_ITEM (g_task_get_source_object (task));
+  priv = self->priv;
+
+  cancellable = g_task_get_cancellable (task);
+
+  error = NULL;
+  buffer = photos_base_item_load_buffer_finish (self, res, &error);
+  if (error != NULL)
+    {
+      g_task_return_error (task, error);
+      goto out;
+    }
+
+  format = gegl_buffer_get_format (buffer);
+  format_name = babl_get_name (format);
+  photos_debug (PHOTOS_DEBUG_GEGL, "Buffer loaded: %s", format_name);
+
+  gegl_node_set (priv->buffer_source, "buffer", buffer, NULL);
+
+  g_clear_object (&priv->processor);
+  priv->processor = photos_pipeline_new_processor (priv->pipeline);
+
+  photos_base_item_process_async (self, cancellable, photos_base_item_load_process, g_object_ref (task));
+
+ out:
+  g_clear_object (&buffer);
+  g_object_unref (task);
+}
+
+
+static gboolean
+photos_base_item_process_idle (gpointer user_data)
+{
+  GTask *task = G_TASK (user_data);
+  PhotosBaseItem *self;
+  GCancellable *cancellable;
+  gboolean result = FALSE;
+
+  self = PHOTOS_BASE_ITEM (g_task_get_source_object (task));
+  cancellable = g_task_get_cancellable (task);
+
+  if (g_cancellable_is_cancelled (cancellable))
+    goto done;
+
+  if (gegl_processor_work (self->priv->processor, NULL))
+    return G_SOURCE_CONTINUE;
+
+  result = TRUE;
+
+ done:
+  g_task_return_boolean (task, result);
+  return G_SOURCE_REMOVE;
+}
+
+
+static void
+photos_base_item_save_save_to_stream (GObject *source_object, GAsyncResult *res, gpointer user_data)
+{
+  GTask *task = G_TASK (user_data);
+  GError *error = NULL;
+  gboolean result;
+
+  result = gdk_pixbuf_save_to_stream_finish (res, &error);
+  if (error != NULL)
+    {
+      g_task_return_error (task, error);
+      goto out;
+    }
+
+  g_task_return_boolean (task, result);
+
+ out:
+  g_object_unref (task);
+}
+
+
+static void
+photos_base_item_save_replace (GObject *source_object, GAsyncResult *res, gpointer user_data)
+{
+  GTask *task = G_TASK (user_data);
+  PhotosBaseItem *self;
+  PhotosBaseItemPrivate *priv;
+  GCancellable *cancellable;
+  GdkPixbuf *pixbuf = NULL;
+  GError *error = NULL;
+  GeglNode *graph;
+  GFile *file = G_FILE (source_object);
+  GFileOutputStream *stream = NULL;
+  const gchar *type;
+
+  self = PHOTOS_BASE_ITEM (g_task_get_source_object (task));
+  priv = self->priv;
+
+  cancellable = g_task_get_cancellable (task);
+  type = (const gchar *) g_task_get_task_data (task);
+
+  stream = g_file_replace_finish (file, res, &error);
+  if (error != NULL)
+    {
+      g_task_return_error (task, error);
+      goto out;
+    }
+
+  if (priv->processor == NULL)
+    {
+      g_task_return_new_error (task, PHOTOS_ERROR, 0, "Failed to find a processor for the GEGL graph");
+      goto out;
+    }
+
+  if (gegl_processor_work (priv->processor, NULL))
+    {
+      g_task_return_new_error (task, PHOTOS_ERROR, 0, "Have not finished processing the GEGL graph");
+      goto out;
+    }
+
+  graph = photos_pipeline_get_graph (priv->pipeline);
+  pixbuf = photos_utils_create_pixbuf_from_node (graph);
+  if (pixbuf == NULL)
+    {
+      g_task_return_new_error (task, PHOTOS_ERROR, 0, "Failed to create a GdkPixbuf from the GEGL graph");
+      goto out;
+    }
+
+  gdk_pixbuf_save_to_stream_async (pixbuf,
+                                   G_OUTPUT_STREAM (stream),
+                                   type,
+                                   cancellable,
+                                   photos_base_item_save_save_to_stream,
+                                   g_object_ref (task),
+                                   NULL);
+
+ out:
+  g_clear_object (&pixbuf);
+  g_clear_object (&stream);
+  g_object_unref (task);
+}
+
+
 static void
 photos_base_item_set_thumbnailing_icon (PhotosBaseItem *self)
 {
@@ -856,6 +1084,7 @@ photos_base_item_populate_from_cursor (PhotosBaseItem *self, TrackerSparqlCursor
   const gchar *equipment;
   const gchar *flash;
   const gchar *mtime;
+  const gchar *orientation;
   const gchar *title;
   const gchar *uri;
 
@@ -907,6 +1136,9 @@ photos_base_item_populate_from_cursor (PhotosBaseItem *self, TrackerSparqlCursor
   equipment = tracker_sparql_cursor_get_string (cursor, PHOTOS_QUERY_COLUMNS_EQUIPMENT, NULL);
   priv->equipment = g_quark_from_string (equipment);
 
+  orientation = tracker_sparql_cursor_get_string (cursor, PHOTOS_QUERY_COLUMNS_ORIENTATION, NULL);
+  priv->orientation = g_quark_from_string (orientation);
+
   priv->exposure_time = tracker_sparql_cursor_get_double (cursor, PHOTOS_QUERY_COLUMNS_EXPOSURE_TIME);
   priv->fnumber = tracker_sparql_cursor_get_double (cursor, PHOTOS_QUERY_COLUMNS_FNUMBER);
   priv->focal_length = tracker_sparql_cursor_get_double (cursor, PHOTOS_QUERY_COLUMNS_FOCAL_LENGTH);
@@ -980,9 +1212,12 @@ photos_base_item_dispose (GObject *object)
   PhotosBaseItemPrivate *priv = self->priv;
 
   g_clear_pointer (&priv->surface, (GDestroyNotify) cairo_surface_destroy);
-  g_clear_object (&priv->graph);
+  g_clear_object (&priv->edit_graph);
+  g_clear_object (&priv->load_graph);
+  g_clear_object (&priv->processor);
   g_clear_object (&priv->original_icon);
   g_clear_object (&priv->watcher);
+  g_clear_object (&priv->pipeline);
   g_clear_object (&priv->sel_cntrlr);
   g_clear_object (&priv->cursor);
 
@@ -1169,7 +1404,6 @@ photos_base_item_download_async (PhotosBaseItem *self,
   GTask *task;
 
   task = g_task_new (self, cancellable, callback, user_data);
-  g_task_set_check_cancellable (task, TRUE);
   g_task_set_source_tag (task, photos_base_item_download_async);
 
   g_task_run_in_thread (task, photos_base_item_download_in_thread_func);
@@ -1197,10 +1431,20 @@ photos_base_item_get_author (PhotosBaseItem *self)
 }
 
 
-void
-photos_base_item_get_bbox (PhotosBaseItem *self, GeglRectangle *out_bbox)
+gboolean
+photos_base_item_get_bbox_source (PhotosBaseItem *self, GeglRectangle *bbox)
 {
-  *out_bbox = self->priv->bbox;
+  PhotosBaseItemPrivate *priv = self->priv;
+  gboolean ret_val = FALSE;
+
+  if (priv->buffer_source == NULL)
+    goto out;
+
+  *bbox = gegl_node_get_bounding_box (priv->buffer_source);
+  ret_val = TRUE;
+
+ out:
+  return ret_val;
 }
 
 
@@ -1405,10 +1649,9 @@ photos_base_item_load_async (PhotosBaseItem *self,
   g_return_if_fail (PHOTOS_IS_BASE_ITEM (self));
 
   task = g_task_new (self, cancellable, callback, user_data);
-  g_task_set_check_cancellable (task, TRUE);
   g_task_set_source_tag (task, photos_base_item_load_async);
 
-  g_task_run_in_thread (task, photos_base_item_load_in_thread_func);
+  photos_base_item_load_buffer_async (self, cancellable, photos_base_item_load_load_buffer, g_object_ref (task));
   g_object_unref (task);
 }
 
@@ -1433,6 +1676,38 @@ photos_base_item_open (PhotosBaseItem *self, GdkScreen *screen, guint32 timestam
 }
 
 
+void
+photos_base_item_operation_add (PhotosBaseItem *self, const gchar *operation, const gchar *first_property_name, ...)
+{
+  va_list ap;
+
+  va_start (ap, first_property_name);
+  photos_pipeline_add (self->priv->pipeline, operation, first_property_name, ap);
+  va_end (ap);
+}
+
+
+gboolean
+photos_base_item_operation_get (PhotosBaseItem *self, const gchar *operation, const gchar *first_property_name, ...)
+{
+  gboolean ret_val;
+  va_list ap;
+
+  va_start (ap, first_property_name);
+  ret_val = photos_pipeline_get (self->priv->pipeline, operation, first_property_name, ap);
+  va_end (ap);
+
+  return ret_val;
+}
+
+
+gboolean
+photos_base_item_operation_undo (PhotosBaseItem *self)
+{
+  return photos_pipeline_undo (self->priv->pipeline);
+}
+
+
 void
 photos_base_item_print (PhotosBaseItem *self, GtkWidget *toplevel)
 {
@@ -1440,6 +1715,37 @@ photos_base_item_print (PhotosBaseItem *self, GtkWidget *toplevel)
 }
 
 
+void
+photos_base_item_process_async (PhotosBaseItem *self,
+                                GCancellable *cancellable,
+                                GAsyncReadyCallback callback,
+                                gpointer user_data)
+{
+  GTask *task;
+
+  g_return_if_fail (PHOTOS_IS_BASE_ITEM (self));
+
+  task = g_task_new (self, cancellable, callback, user_data);
+  g_task_set_source_tag (task, photos_base_item_process_async);
+
+  g_idle_add_full (G_PRIORITY_DEFAULT_IDLE, photos_base_item_process_idle, g_object_ref (task), g_object_unref);
+  g_object_unref (task);
+}
+
+
+gboolean
+photos_base_item_process_finish (PhotosBaseItem *self, GAsyncResult *res, GError **error)
+{
+  GTask *task = G_TASK (res);
+
+  g_return_val_if_fail (g_task_is_valid (res, self), FALSE);
+  g_return_val_if_fail (g_task_get_source_tag (task) == photos_base_item_process_async, FALSE);
+  g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
+
+  return g_task_propagate_boolean (task, error);
+}
+
+
 void
 photos_base_item_refresh (PhotosBaseItem *self)
 {
@@ -1460,6 +1766,58 @@ photos_base_item_refresh (PhotosBaseItem *self)
 }
 
 
+void
+photos_base_item_save_async (PhotosBaseItem *self,
+                             const gchar *uri,
+                             const gchar *type,
+                             GCancellable *cancellable,
+                             GAsyncReadyCallback callback,
+                             gpointer user_data)
+{
+  PhotosBaseItemPrivate *priv = self->priv;
+  GFile *file;
+  GTask *task;
+
+  g_return_if_fail (PHOTOS_IS_BASE_ITEM (self));
+  g_return_if_fail (uri != NULL && uri[0] != '\0');
+  g_return_if_fail (type != NULL && type[0] != '\0');
+  g_return_if_fail (priv->edit_graph != NULL);
+  g_return_if_fail (priv->load_graph != NULL);
+  g_return_if_fail (priv->processor != NULL);
+  g_return_if_fail (!gegl_processor_work (priv->processor, NULL));
+
+  task = g_task_new (self, cancellable, callback, user_data);
+  g_task_set_source_tag (task, photos_base_item_save_async);
+  g_task_set_task_data (task, g_strdup (type), g_free);
+
+  file = g_file_new_for_uri (uri);
+  g_file_replace_async (file,
+                        NULL,
+                        TRUE,
+                        G_FILE_CREATE_REPLACE_DESTINATION,
+                        G_PRIORITY_DEFAULT,
+                        cancellable,
+                        photos_base_item_save_replace,
+                        g_object_ref (task));
+
+  g_object_unref (file);
+  g_object_unref (task);
+}
+
+
+gboolean
+photos_base_item_save_finish (PhotosBaseItem *self, GAsyncResult *res, GError **error)
+{
+  GTask *task = G_TASK (res);
+
+  g_return_val_if_fail (g_task_is_valid (res, self), FALSE);
+  g_return_val_if_fail (g_task_get_source_tag (task) == photos_base_item_save_async, FALSE);
+  g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
+
+  return g_task_propagate_boolean (task, error);
+}
+
+
 void
 photos_base_item_set_default_app_name (PhotosBaseItem *self, const gchar *default_app_name)
 {
