@@ -108,7 +108,7 @@ gchar              *photos_base_item_download_finish         (PhotosBaseItem *se
 
 const gchar        *photos_base_item_get_author              (PhotosBaseItem *self);
 
-void                photos_base_item_get_bbox                (PhotosBaseItem *self, GeglRectangle *out_bbox);
+gboolean            photos_base_item_get_bbox_source         (PhotosBaseItem *self, GeglRectangle *bbox);
 
 gint64              photos_base_item_get_date_created        (PhotosBaseItem *self);
 
@@ -173,10 +173,42 @@ void                photos_base_item_open                    (PhotosBaseItem *se
                                                               GdkScreen *screen,
                                                               guint32 timestamp);
 
+void                photos_base_item_operation_add           (PhotosBaseItem *self,
+                                                              const gchar *operation,
+                                                              const gchar *first_property_name,
+                                                              ...) G_GNUC_NULL_TERMINATED;
+
+gboolean            photos_base_item_operation_get           (PhotosBaseItem *self,
+                                                              const gchar *operation,
+                                                              const gchar *first_property_name,
+                                                              ...) G_GNUC_NULL_TERMINATED G_GNUC_WARN_UNUSED_RESULT;
+
+gboolean            photos_base_item_operation_undo          (PhotosBaseItem *self);
+
 void                photos_base_item_print                   (PhotosBaseItem *self, GtkWidget *toplevel);
 
+void                photos_base_item_process_async           (PhotosBaseItem *self,
+                                                              GCancellable *cancellable,
+                                                              GAsyncReadyCallback callback,
+                                                              gpointer user_data);
+
+gboolean            photos_base_item_process_finish          (PhotosBaseItem *self,
+                                                              GAsyncResult *res,
+                                                              GError **error);
+
 void                photos_base_item_refresh                 (PhotosBaseItem *self);
 
+void                photos_base_item_save_async              (PhotosBaseItem *self,
+                                                              const gchar *uri,
+                                                              const gchar *type,
+                                                              GCancellable *cancellable,
+                                                              GAsyncReadyCallback callback,
+                                                              gpointer user_data);
+
+gboolean            photos_base_item_save_finish             (PhotosBaseItem *self,
+                                                              GAsyncResult *res,
+                                                              GError **error);
+
 void                photos_base_item_set_default_app_name    (PhotosBaseItem *self, const gchar *default_app_name);
 
 void                photos_base_item_set_favorite            (PhotosBaseItem *self, gboolean favorite);
