@@ -45,7 +45,6 @@
 struct _PhotosMainToolbarPrivate
 {
   GAction *search;
-  GCancellable *loader_cancellable;
   GSimpleAction *gear_menu;
   GtkWidget *coll_back_button;
   GtkWidget *favorite_button;
@@ -125,7 +124,7 @@ photos_main_toolbar_set_toolbar_title (PhotosMainToolbar *self)
           g_free (label);
         }
     }
-  else if (window_mode == PHOTOS_WINDOW_MODE_PREVIEW)
+  else if (window_mode == PHOTOS_WINDOW_MODE_EDIT || window_mode == PHOTOS_WINDOW_MODE_PREVIEW)
     {
       GObject *item;
 
@@ -335,12 +334,7 @@ photos_main_toolbar_add_selection_button (PhotosMainToolbar *self)
 static void
 photos_main_toolbar_back_button_clicked (PhotosMainToolbar *self)
 {
-  PhotosMainToolbarPrivate *priv = self->priv;
-
-  if (priv->loader_cancellable != NULL)
-    g_cancellable_cancel (priv->loader_cancellable);
-
-  photos_mode_controller_go_back (priv->mode_cntrlr);
+  photos_mode_controller_go_back (self->priv->mode_cntrlr);
 }
 
 
@@ -401,8 +395,7 @@ photos_main_toolbar_create_preview_menu (PhotosMainToolbar *self)
   GtkBuilder *builder;
   PhotosBaseItem *item;
 
-  builder = gtk_builder_new ();
-  gtk_builder_add_from_resource (builder, "/org/gnome/Photos/preview-menu.ui", NULL);
+  builder = gtk_builder_new_from_resource ("/org/gnome/Photos/preview-menu.ui");
 
   menu = G_MENU (g_object_ref (gtk_builder_get_object (builder, "preview-menu")));
   item = PHOTOS_BASE_ITEM (photos_base_manager_get_active_object (self->priv->item_mngr));
@@ -494,6 +487,28 @@ photos_main_toolbar_populate_for_collections (PhotosMainToolbar *self)
 }
 
 
+static void
+photos_main_toolbar_populate_for_edit (PhotosMainToolbar *self)
+{
+  PhotosMainToolbarPrivate *priv = self->priv;
+  GtkStyleContext *context;
+  GtkWidget *cancel_button;
+  GtkWidget *done_button;
+
+  photos_header_bar_set_mode (PHOTOS_HEADER_BAR (priv->toolbar), PHOTOS_HEADER_BAR_MODE_STANDALONE);
+
+  cancel_button = gtk_button_new_with_label (_("Cancel"));
+  gtk_actionable_set_action_name (GTK_ACTIONABLE (cancel_button), "app.edit-cancel");
+  gtk_header_bar_pack_start (GTK_HEADER_BAR (priv->toolbar), cancel_button);
+
+  done_button = gtk_button_new_with_label (_("Done"));
+  gtk_actionable_set_action_name (GTK_ACTIONABLE (done_button), "app.save-current");
+  gtk_header_bar_pack_end (GTK_HEADER_BAR (priv->toolbar), done_button);
+  context = gtk_widget_get_style_context (done_button);
+  gtk_style_context_add_class (context, "suggested-action");
+}
+
+
 static void
 photos_main_toolbar_populate_for_favorites (PhotosMainToolbar *self)
 {
@@ -538,6 +553,7 @@ photos_main_toolbar_populate_for_preview (PhotosMainToolbar *self)
   PhotosMainToolbarPrivate *priv = self->priv;
   GMenu *preview_menu;
   GtkWidget *back_button;
+  GtkWidget *edit_button;
   GtkWidget *image;
   GtkWidget *menu_button;
   GApplication *app;
@@ -562,6 +578,10 @@ photos_main_toolbar_populate_for_preview (PhotosMainToolbar *self)
 
   g_simple_action_set_enabled (priv->gear_menu, TRUE);
 
+  edit_button = gtk_button_new_from_icon_name (PHOTOS_ICON_IMAGE_EDIT_SYMBOLIC, GTK_ICON_SIZE_BUTTON);
+  gtk_actionable_set_action_name (GTK_ACTIONABLE (edit_button), "app.edit-current");
+  gtk_header_bar_pack_end (GTK_HEADER_BAR (priv->toolbar), edit_button);
+
   priv->favorite_button = gtk_button_new ();
   gtk_header_bar_pack_end (GTK_HEADER_BAR (priv->toolbar), priv->favorite_button);
   g_signal_connect_swapped (priv->favorite_button,
@@ -658,7 +678,6 @@ photos_main_toolbar_dispose (GObject *object)
 
   photos_main_toolbar_clear_state_data (self);
 
-  g_clear_object (&priv->loader_cancellable);
   g_clear_object (&priv->searchbar);
   g_clear_object (&priv->item_mngr);
   g_clear_object (&priv->mode_cntrlr);
@@ -735,8 +754,7 @@ photos_main_toolbar_init (PhotosMainToolbar *self)
   priv->gear_menu = G_SIMPLE_ACTION (g_action_map_lookup_action (G_ACTION_MAP (app), "gear-menu"));
   priv->search = g_action_map_lookup_action (G_ACTION_MAP (app), "search");
 
-  builder = gtk_builder_new ();
-  gtk_builder_add_from_resource (builder, "/org/gnome/Photos/selection-menu.ui", NULL);
+  builder = gtk_builder_new_from_resource ("/org/gnome/Photos/selection-menu.ui");
 
   selection_menu = G_MENU (gtk_builder_get_object (builder, "selection-menu"));
   priv->selection_menu = gtk_menu_button_new ();
@@ -840,6 +858,8 @@ photos_main_toolbar_reset_toolbar_mode (PhotosMainToolbar *self)
     photos_main_toolbar_populate_for_selection_mode (self);
   else if (window_mode == PHOTOS_WINDOW_MODE_COLLECTIONS)
     photos_main_toolbar_populate_for_collections (self);
+  else if (window_mode == PHOTOS_WINDOW_MODE_EDIT)
+    photos_main_toolbar_populate_for_edit (self);
   else if (window_mode == PHOTOS_WINDOW_MODE_FAVORITES)
     photos_main_toolbar_populate_for_favorites (self);
   else if (window_mode == PHOTOS_WINDOW_MODE_OVERVIEW)
@@ -856,23 +876,6 @@ photos_main_toolbar_reset_toolbar_mode (PhotosMainToolbar *self)
 }
 
 
-void
-photos_main_toolbar_set_cancellable (PhotosMainToolbar *self, GCancellable *cancellable)
-{
-  PhotosMainToolbarPrivate *priv = self->priv;
-
-  if (cancellable == priv->loader_cancellable)
-    return;
-
-  g_clear_object (&priv->loader_cancellable);
-
-  if (cancellable != NULL)
-    g_object_ref (cancellable);
-
-  priv->loader_cancellable = cancellable;
-}
-
-
 void
 photos_main_toolbar_set_stack (PhotosMainToolbar *self, GtkStack *stack)
 {
