@@ -1,6 +1,7 @@
 /*
  * Photos - access, organize and share your photos on GNOME
  * Copyright © 2012, 2013, 2014, 2015 Red Hat, Inc.
+ * Copyright © 2009 Yorba Foundation
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -21,6 +22,7 @@
 /* Based on code from:
  *   + Documents
  *   + Eye of GNOME
+ *   + Shotwell
  *   + Totem
  */
 
@@ -29,7 +31,6 @@
 
 #include <string.h>
 
-#include <cairo.h>
 #include <glib.h>
 #include <libgnome-desktop/gnome-desktop-thumbnail.h>
 #include <tracker-sparql.h>
@@ -41,8 +42,15 @@
 #include "photos-google-item.h"
 #include "photos-local-item.h"
 #include "photos-media-server-item.h"
+#include "photos-operation-insta-curve.h"
+#include "photos-operation-insta-filter.h"
 #include "photos-query.h"
 #include "photos-source.h"
+#include "photos-tool.h"
+#include "photos-tool-colors.h"
+#include "photos-tool-crop.h"
+#include "photos-tool-enhance.h"
+#include "photos-tool-filters.h"
 #include "photos-tracker-collections-controller.h"
 #include "photos-tracker-controller.h"
 #include "photos-tracker-favorites-controller.h"
@@ -52,7 +60,7 @@
 #include "photos-utils.h"
 
 
-static const gchar *dot_dir;
+static const gdouble EPSILON = 1e-5;
 
 
 static void
@@ -117,6 +125,23 @@ photos_utils_center_pixbuf (GdkPixbuf *pixbuf, gint size)
 }
 
 
+gchar *
+photos_utils_convert_path_to_uri (const gchar *path)
+{
+  GFile *file;
+  gchar *uri;
+
+  if (path == NULL)
+    return g_strdup ("");
+
+  file = g_file_new_for_path (path);
+  uri = g_file_get_uri (file);
+  g_object_unref (file);
+
+  return uri;
+}
+
+
 GIcon *
 photos_utils_create_collection_icon (gint base_size, GList *pixbufs)
 {
@@ -218,13 +243,42 @@ photos_utils_create_collection_icon (gint base_size, GList *pixbufs)
 }
 
 
+GeglNode *
+photos_utils_create_orientation_node (GeglNode *parent, GQuark orientation)
+{
+  GeglNode *ret_val = NULL;
+  double degrees = 1.0;
+
+  if (orientation == PHOTOS_ORIENTATION_TOP)
+    goto out;
+
+  if (orientation == PHOTOS_ORIENTATION_BOTTOM)
+    degrees = -180.0;
+  else if (orientation == PHOTOS_ORIENTATION_LEFT)
+    degrees = -270.0;
+  else if (orientation == PHOTOS_ORIENTATION_RIGHT)
+    degrees = -90.0;
+
+  if (degrees < 0.0)
+    ret_val = gegl_node_new_child (parent, "operation", "gegl:rotate-on-center", "degrees", degrees, NULL);
+
+ out:
+  if (ret_val == NULL)
+    ret_val = gegl_node_new_child (parent, "operation", "gegl:nop", NULL);
+
+  return ret_val;
+}
+
+
 GdkPixbuf *
 photos_utils_create_pixbuf_from_node (GeglNode *node)
 {
   GdkPixbuf *pixbuf = NULL;
+  GeglNode *graph;
   GeglNode *save_pixbuf;
 
-  save_pixbuf = gegl_node_new_child (gegl_node_get_parent (node),
+  graph = gegl_node_get_parent (node);
+  save_pixbuf = gegl_node_new_child (graph,
                                      "operation", "gegl:save-pixbuf",
                                      "pixbuf", &pixbuf,
                                      NULL);
@@ -237,7 +291,7 @@ photos_utils_create_pixbuf_from_node (GeglNode *node)
 
 
 GIcon *
-photos_utils_create_symbolic_icon (const gchar *name, gint base_size)
+photos_utils_create_symbolic_icon_for_scale (const gchar *name, gint base_size, gint scale)
 {
   GIcon *icon;
   GIcon *ret_val = NULL;
@@ -246,6 +300,7 @@ photos_utils_create_symbolic_icon (const gchar *name, gint base_size)
   GtkIconTheme *theme;
   GtkStyleContext *style;
   GtkWidgetPath *path;
+  cairo_surface_t *icon_surface = NULL;
   cairo_surface_t *surface;
   cairo_t *cr;
   gchar *symbolic_name;
@@ -254,11 +309,14 @@ photos_utils_create_symbolic_icon (const gchar *name, gint base_size)
   gint emblem_pos;
   gint emblem_size;
   gint total_size;
+  gint total_size_scaled;
 
   total_size = base_size / 2;
+  total_size_scaled = total_size * scale;
   emblem_size = bg_size - emblem_margin * 2;
 
-  surface = cairo_image_surface_create (CAIRO_FORMAT_ARGB32, total_size, total_size);
+  surface = cairo_image_surface_create (CAIRO_FORMAT_ARGB32, total_size_scaled, total_size_scaled);
+  cairo_surface_set_device_scale (surface, (gdouble) scale, (gdouble) scale);
   cr = cairo_create (surface);
 
   style = gtk_style_context_new ();
@@ -277,7 +335,7 @@ photos_utils_create_symbolic_icon (const gchar *name, gint base_size)
   g_free (symbolic_name);
 
   theme = gtk_icon_theme_get_default();
-  info = gtk_icon_theme_lookup_by_gicon (theme, icon, emblem_size, GTK_ICON_LOOKUP_FORCE_SIZE);
+  info = gtk_icon_theme_lookup_by_gicon_for_scale (theme, icon, emblem_size, scale, GTK_ICON_LOOKUP_FORCE_SIZE);
   g_object_unref (icon);
 
   if (info == NULL)
@@ -289,14 +347,17 @@ photos_utils_create_symbolic_icon (const gchar *name, gint base_size)
   if (pixbuf == NULL)
     goto out;
 
-  emblem_pos = total_size - emblem_size - emblem_margin;
-  gtk_render_icon (style, cr, pixbuf, emblem_pos, emblem_pos);
+  icon_surface = gdk_cairo_surface_create_from_pixbuf (pixbuf, scale, NULL);
   g_object_unref (pixbuf);
 
-  ret_val = G_ICON (gdk_pixbuf_get_from_surface (surface, 0, 0, total_size, total_size));
+  emblem_pos = total_size - emblem_size - emblem_margin;
+  gtk_render_icon_surface (style, cr, icon_surface, emblem_pos, emblem_pos);
+
+  ret_val = G_ICON (gdk_pixbuf_get_from_surface (surface, 0, 0, total_size_scaled, total_size_scaled));
 
  out:
   g_object_unref (style);
+  cairo_surface_destroy (icon_surface);
   cairo_surface_destroy (surface);
   cairo_destroy (cr);
 
@@ -430,27 +491,6 @@ photos_utils_get_icon_from_cursor (TrackerSparqlCursor *cursor)
 }
 
 
-const gchar *
-photos_utils_dot_dir (void)
-{
-  const gchar *config_dir;
-
-  if (dot_dir == NULL)
-    {
-      config_dir = g_get_user_config_dir ();
-      dot_dir = g_build_filename (config_dir, PACKAGE_TARNAME, NULL);
-    }
-
-  if (g_file_test (dot_dir, G_FILE_TEST_IS_DIR))
-    goto out;
-
-  g_mkdir_with_parents (dot_dir, 0700);
-
- out:
-  return dot_dir;
-}
-
-
 GdkPixbuf *
 photos_utils_downscale_pixbuf_for_scale (GdkPixbuf *pixbuf, gint size, gint scale)
 {
@@ -520,6 +560,65 @@ photos_utils_downscale_pixbuf_for_scale (GdkPixbuf *pixbuf, gint size, gint scal
 }
 
 
+void
+photos_utils_draw_rectangle_handles (cairo_t *cr,
+                                     gdouble x,
+                                     gdouble y,
+                                     gdouble width,
+                                     gdouble height,
+                                     gdouble offset,
+                                     gdouble radius)
+{
+  cairo_save (cr);
+
+  cairo_new_sub_path (cr);
+  cairo_arc (cr, x - offset, y - offset, radius, 0.0, 2.0 * M_PI);
+  cairo_fill (cr);
+
+  cairo_new_sub_path (cr);
+  cairo_arc (cr, x + width + offset, y - offset, radius, 0.0, 2.0 * M_PI);
+  cairo_fill (cr);
+
+  cairo_new_sub_path (cr);
+  cairo_arc (cr, x + width + offset, y + height + offset, radius, 0.0, 2.0 * M_PI);
+  cairo_fill (cr);
+
+  cairo_new_sub_path (cr);
+  cairo_arc (cr, x - offset, y + height + offset, radius, 0.0, 2.0 * M_PI);
+  cairo_fill (cr);
+
+  cairo_restore (cr);
+}
+
+
+void
+photos_utils_draw_rectangle_thirds (cairo_t *cr, gdouble x, gdouble y, gdouble width, gdouble height)
+{
+  const gdouble one_third_x = width / 3.0;
+  const gdouble one_third_y = height / 3.0;
+
+  cairo_save (cr);
+
+  cairo_move_to (cr, x + one_third_x, y);
+  cairo_line_to (cr, x + one_third_x, y + height);
+  cairo_stroke (cr);
+
+  cairo_move_to (cr, x + 2.0 * one_third_x, y);
+  cairo_line_to (cr, x + 2.0 * one_third_x, y + height);
+  cairo_stroke (cr);
+
+  cairo_move_to (cr, x, y + one_third_y);
+  cairo_line_to (cr, x + width, y + one_third_y);
+  cairo_stroke (cr);
+
+  cairo_move_to (cr, x, y + 2.0 * one_third_y);
+  cairo_line_to (cr, x + width, y + 2.0 * one_third_y);
+  cairo_stroke (cr);
+
+  cairo_restore (cr);
+}
+
+
 void
 photos_utils_ensure_builtins (void)
 {
@@ -535,6 +634,14 @@ photos_utils_ensure_builtins (void)
       g_type_ensure (PHOTOS_TYPE_LOCAL_ITEM);
       g_type_ensure (PHOTOS_TYPE_MEDIA_SERVER_ITEM);
 
+      g_type_ensure (PHOTOS_TYPE_OPERATION_INSTA_CURVE);
+      g_type_ensure (PHOTOS_TYPE_OPERATION_INSTA_FILTER);
+
+      g_type_ensure (PHOTOS_TYPE_TOOL_COLORS);
+      g_type_ensure (PHOTOS_TYPE_TOOL_CROP);
+      g_type_ensure (PHOTOS_TYPE_TOOL_ENHANCE);
+      g_type_ensure (PHOTOS_TYPE_TOOL_FILTERS);
+
       g_type_ensure (PHOTOS_TYPE_TRACKER_COLLECTIONS_CONTROLLER);
       g_type_ensure (PHOTOS_TYPE_TRACKER_FAVORITES_CONTROLLER);
       g_type_ensure (PHOTOS_TYPE_TRACKER_OVERVIEW_CONTROLLER);
@@ -557,6 +664,9 @@ photos_utils_ensure_extension_points (void)
       extension_point = g_io_extension_point_register (PHOTOS_BASE_ITEM_EXTENSION_POINT_NAME);
       g_io_extension_point_set_required_type (extension_point, PHOTOS_TYPE_BASE_ITEM);
 
+      extension_point = g_io_extension_point_register (PHOTOS_TOOL_EXTENSION_POINT_NAME);
+      g_io_extension_point_set_required_type (extension_point, PHOTOS_TYPE_TOOL);
+
       extension_point = g_io_extension_point_register (PHOTOS_TRACKER_CONTROLLER_EXTENSION_POINT_NAME);
       g_io_extension_point_set_required_type (extension_point, PHOTOS_TYPE_TRACKER_CONTROLLER);
 
@@ -565,6 +675,14 @@ photos_utils_ensure_extension_points (void)
 }
 
 
+gboolean
+photos_utils_equal_double (gdouble a, gdouble b)
+{
+  const gdouble diff = a - b;
+  return diff > -EPSILON && diff < EPSILON;
+}
+
+
 GQuark
 photos_utils_error_quark (void)
 {
@@ -572,6 +690,25 @@ photos_utils_error_quark (void)
 }
 
 
+gdouble
+photos_utils_eval_radial_line (gdouble crop_center_x,
+                               gdouble crop_center_y,
+                               gdouble corner_x,
+                               gdouble corner_y,
+                               gdouble event_x)
+{
+  gdouble decision_intercept;
+  gdouble decision_slope;
+  gdouble projected_y;
+
+  decision_slope = (corner_y - crop_center_y) / (corner_x - crop_center_x);
+  decision_intercept = corner_y - (decision_slope * corner_x);
+  projected_y = decision_slope * event_x + decision_intercept;
+
+  return projected_y;
+}
+
+
 static gchar *
 photos_utils_filename_get_extension_offset (const gchar *filename)
 {
@@ -722,111 +859,38 @@ photos_utils_get_pixbuf_common_suffix (GdkPixbufFormat *format)
 }
 
 
-static gchar *
-photos_utils_get_pixbuf_suffix_from_basename (const gchar *basename)
+gchar *
+photos_utils_get_pixbuf_type_from_mime_type (const gchar *mime_type)
 {
-  gchar *suffix;
-  gchar *suffix_start;
-  guint len;
+  GSList *formats;
+  GSList *l;
+  gchar *ret_val = NULL;
 
-  /* FIXME: does this work for all locales? */
-  suffix_start = g_utf8_strrchr (basename, -1, '.');
+  formats = gdk_pixbuf_get_formats ();
 
-  if (suffix_start == NULL)
-    return NULL;
-
-  len = strlen (suffix_start) - 1;
-  suffix = g_strndup (suffix_start+1, len);
-
-  return suffix;
-}
-
-
-GdkPixbufFormat *
-photos_utils_get_pixbuf_format (GFile *file)
-{
-  GdkPixbufFormat *format;
-  gchar *basename;
-  gchar *path;
-  gchar *suffix;
-
-  g_return_val_if_fail (file != NULL, NULL);
-
-  path = g_file_get_path (file);
-  basename = g_path_get_basename (path);
-  suffix = photos_utils_get_pixbuf_suffix_from_basename (basename);
-
-  format = photos_utils_get_pixbuf_format_by_suffix (suffix);
-
-  g_free (path);
-  g_free (basename);
-  g_free (suffix);
-
-  return format;
-}
-
-
-GdkPixbufFormat*
-photos_utils_get_pixbuf_format_by_suffix (const gchar *suffix)
-{
-  GSList *list;
-  GSList *it;
-  GdkPixbufFormat *result = NULL;
-
-  g_return_val_if_fail (suffix != NULL, NULL);
-
-  list = gdk_pixbuf_get_formats ();
-
-  for (it = list; (it != NULL) && (result == NULL); it = it->next)
+  for (l = formats; l != NULL; l = l->next)
     {
-      GdkPixbufFormat *format;
-      gchar **extensions;
-      gint i;
+      GdkPixbufFormat *format = (GdkPixbufFormat*) l->data;
+      gchar **supported_mime_types;
+      guint i;
 
-      format = (GdkPixbufFormat*) it->data;
-
-      extensions = gdk_pixbuf_format_get_extensions (format);
-      for (i = 0; extensions[i] != NULL; i++)
+      supported_mime_types = gdk_pixbuf_format_get_mime_types (format);
+      for (i = 0; supported_mime_types[i] != NULL; i++)
         {
-          /* g_print ("check extension: %s against %s\n", extensions[i], suffix); */
-          if (g_ascii_strcasecmp (suffix, extensions[i]) == 0)
+          if (g_strcmp0 (mime_type, supported_mime_types[i]) == 0)
             {
-              result = format;
+              ret_val = gdk_pixbuf_format_get_name (format);
               break;
-          }
+            }
         }
 
-      g_strfreev (extensions);
+      g_strfreev (supported_mime_types);
+      if (ret_val != NULL)
+        break;
     }
 
-  g_slist_free (list);
-
-  return result;
-}
-
-
-GSList *
-photos_utils_get_pixbuf_savable_formats (void)
-{
-  GSList *list;
-  GSList *write_list = NULL;
-  GSList *it;
-
-  list = gdk_pixbuf_get_formats ();
-
-  for (it = list; it != NULL; it = it->next)
-    {
-      GdkPixbufFormat *format;
-
-      format = (GdkPixbufFormat*) it->data;
-      if (gdk_pixbuf_format_is_writable (format))
-        write_list = g_slist_prepend (write_list, format);
-    }
-
-  g_slist_free (list);
-  write_list = g_slist_reverse (write_list);
-
-  return write_list;
+  g_slist_free (formats);
+  return ret_val;
 }
 
 
@@ -896,6 +960,34 @@ photos_utils_icon_from_rdf_type (const gchar *type)
 }
 
 
+GQuark
+photos_utils_orientation_bottom_quark (void)
+{
+  return g_quark_from_static_string ("http://www.semanticdesktop.org/ontologies/2007/03/22/nfo#orientation-bottom");
+}
+
+
+GQuark
+photos_utils_orientation_left_quark (void)
+{
+  return g_quark_from_static_string ("http://www.semanticdesktop.org/ontologies/2007/03/22/nfo#orientation-left");
+}
+
+
+GQuark
+photos_utils_orientation_right_quark (void)
+{
+  return g_quark_from_static_string ("http://www.semanticdesktop.org/ontologies/2007/03/22/nfo#orientation-right");
+}
+
+
+GQuark
+photos_utils_orientation_top_quark (void)
+{
+  return g_quark_from_static_string ("http://www.semanticdesktop.org/ontologies/2007/03/22/nfo#orientation-top");
+}
+
+
 static void
 photos_utils_update_executed (GObject *source_object, GAsyncResult *res, gpointer user_data)
 {
