@@ -21,12 +21,14 @@
 /* Based on code from:
  *   + Documents
  *   + Eye of GNOME
+ *   + Shotwell
  *   + Totem
  */
 
 #ifndef PHOTOS_UTILS_H
 #define PHOTOS_UTILS_H
 
+#include <cairo.h>
 #include <gdk-pixbuf/gdk-pixbuf.h>
 #include <gegl.h>
 #include <gio/gio.h>
@@ -42,8 +44,13 @@ G_BEGIN_DECLS
 #define PHOTOS_ERROR (photos_utils_error_quark ())
 #define PHOTOS_FLASH_OFF (photos_utils_flash_off_quark ())
 #define PHOTOS_FLASH_ON (photos_utils_flash_on_quark ())
+#define PHOTOS_ORIENTATION_BOTTOM (photos_utils_orientation_bottom_quark ())
+#define PHOTOS_ORIENTATION_LEFT (photos_utils_orientation_left_quark ())
+#define PHOTOS_ORIENTATION_RIGHT (photos_utils_orientation_right_quark ())
+#define PHOTOS_ORIENTATION_TOP (photos_utils_orientation_top_quark ())
 
 #define PHOTOS_BASE_ITEM_EXTENSION_POINT_NAME "photos-base-item"
+#define PHOTOS_TOOL_EXTENSION_POINT_NAME "photos-tool"
 #define PHOTOS_TRACKER_CONTROLLER_EXTENSION_POINT_NAME "photos-tracker-controller"
 
 #define PHOTOS_COLLECTION_SCREENSHOT \
@@ -53,26 +60,50 @@ void             photos_utils_border_pixbuf               (GdkPixbuf *pixbuf);
 
 GdkPixbuf       *photos_utils_center_pixbuf               (GdkPixbuf *pixbuf, gint size);
 
+gchar           *photos_utils_convert_path_to_uri         (const gchar *path);
+
 GIcon           *photos_utils_create_collection_icon      (gint base_size, GList *pixbufs);
 
+GeglNode        *photos_utils_create_orientation_node     (GeglNode *parent, GQuark orientation);
+
 GdkPixbuf       *photos_utils_create_pixbuf_from_node     (GeglNode *node);
 
-GIcon           *photos_utils_create_symbolic_icon        (const gchar *name, gint base_size);
+GIcon           *photos_utils_create_symbolic_icon_for_scale (const gchar *name, gint base_size, gint scale);
 
 gboolean         photos_utils_create_thumbnail            (GFile *file, GCancellable *cancellable, GError **error);
 
 GIcon           *photos_utils_get_icon_from_cursor        (TrackerSparqlCursor *cursor);
 
-const gchar     *photos_utils_dot_dir                     (void);
-
 GdkPixbuf       *photos_utils_downscale_pixbuf_for_scale  (GdkPixbuf *pixbuf, gint size, gint scale);
 
+void             photos_utils_draw_rectangle_handles      (cairo_t *cr,
+                                                           gdouble x,
+                                                           gdouble y,
+                                                           gdouble width,
+                                                           gdouble height,
+                                                           gdouble offset,
+                                                           gdouble radius);
+
+void             photos_utils_draw_rectangle_thirds       (cairo_t *cr,
+                                                           gdouble x,
+                                                           gdouble y,
+                                                           gdouble width,
+                                                           gdouble height);
+
 void             photos_utils_ensure_builtins             (void);
 
 void             photos_utils_ensure_extension_points     (void);
 
+gboolean         photos_utils_equal_double                (gdouble a, gdouble b);
+
 GQuark           photos_utils_error_quark                 (void);
 
+gdouble          photos_utils_eval_radial_line            (gdouble crop_center_x,
+                                                           gdouble crop_center_y,
+                                                           gdouble corner_x,
+                                                           gdouble corner_y,
+                                                           gdouble event_x);
+
 gchar           *photos_utils_filename_strip_extension    (const gchar *filename_with_extension);
 
 GQuark           photos_utils_flash_off_quark             (void);
@@ -87,11 +118,7 @@ gint             photos_utils_get_icon_size_unscaled      (void);
 
 char*            photos_utils_get_pixbuf_common_suffix    (GdkPixbufFormat *format);
 
-GdkPixbufFormat* photos_utils_get_pixbuf_format           (GFile *file);
-
-GdkPixbufFormat* photos_utils_get_pixbuf_format_by_suffix (const char *suffix);
-
-GSList*          photos_utils_get_pixbuf_savable_formats  (void);
+gchar           *photos_utils_get_pixbuf_type_from_mime_type (const gchar *mime_type);
 
 const gchar     *photos_utils_get_provider_name           (PhotosBaseManager *src_mngr, PhotosBaseItem *item);
 
@@ -101,6 +128,14 @@ GList           *photos_utils_get_urns_from_paths         (GList *paths, GtkTree
 
 GIcon           *photos_utils_icon_from_rdf_type          (const gchar *type);
 
+GQuark           photos_utils_orientation_bottom_quark    (void);
+
+GQuark           photos_utils_orientation_left_quark      (void);
+
+GQuark           photos_utils_orientation_right_quark     (void);
+
+GQuark           photos_utils_orientation_top_quark       (void);
+
 void             photos_utils_set_edited_name             (const gchar *urn, const gchar *title);
 
 void             photos_utils_set_favorite                (const gchar *urn, gboolean is_favorite);
